import traci
import csv
import os
import multiprocessing

def runSimulation(sumoCmd, csv_filename):
    traci.start(sumoCmd)

    # Ensure output directory exists
    os.makedirs(os.path.dirname(csv_filename), exist_ok=True)

    with open(csv_filename, mode="w", newline="") as file:
        writer = csv.writer(file)
        writer.writerow(["TimeStep", "VehicleID", "TimeLoss", "WaitingTime", "SpeedFactor", "MaxSpeed", "Speed",
                         "Vzul", "Acceleration", "CO2", "CO", "Fuel", "DistanceTravelled", "Lon", "Lat"])

        while traci.simulation.getMinExpectedNumber() > 0:  # Run while vehicles exist
            traci.simulationStep()
            timestep = traci.simulation.getTime()
            vehicle_ids = traci.vehicle.getIDList()

            for veh_id in vehicle_ids:
                time_loss = traci.vehicle.getTimeLoss(veh_id)
                waiting_time = traci.vehicle.getWaitingTime(veh_id)
                speed_factor = traci.vehicle.getSpeedFactor(veh_id)
                max_speed = traci.vehicle.getMaxSpeed(veh_id)
                speed = traci.vehicle.getSpeed(veh_id)
                acceleration = traci.vehicle.getAcceleration(veh_id)
                co2 = traci.vehicle.getCO2Emission(veh_id)
                co = traci.vehicle.getCOEmission(veh_id)
                fuel = traci.vehicle.getFuelConsumption(veh_id)
                vzul = traci.vehicle.getAllowedSpeed(veh_id)
                distanceTravelled = traci.vehicle.getDistance(veh_id)
                position = traci.vehicle.getPosition(veh_id)

                # Convert (x, y) to (lon, lat)
                lon, lat = traci.simulation.convertGeo(position[0], position[1])

                # Write data to CSV
                writer.writerow([timestep, veh_id, time_loss, waiting_time, speed_factor, max_speed, speed, vzul,
                                 acceleration, co2, co, fuel, distanceTravelled, lon, lat])

    traci.close()
    print(f"Data saved to {csv_filename}")

def run_simulation_for_rate(args):
    """Runs the SUMO simulation for a specific seed and equipment rate."""
    seed, rate = args
    print(f"Running simulation with seed {seed} and equipment rate {rate}")
    sumoCmd = ["sumo", "-c", "test.sumocfg", "--seed", str(seed), "--device.glosa.probability", str(rate)]
    csv_filename = f"_results/{rate}_{seed}_test_traciResults.csv"
    runSimulation(sumoCmd, csv_filename)

# Define SUMO command
seeds = [924]
equipmentRates = [0, 1.0]

if __name__ == "__main__":
    num_workers = 6
    tasks = [(seed, rate) for seed in seeds for rate in equipmentRates]

    with multiprocessing.Pool(processes=num_workers) as pool:
        pool.map(run_simulation_for_rate, tasks)

    print("All simulations completed.")
